@@ -17,7 +17,7 @@ from camd import tqdm
 from qmpy.analysis.thermodynamics.phase import Phase, PhaseData
 from qmpy.analysis.thermodynamics.space import PhaseSpace
 from multiprocessing import Pool, cpu_count
-from pymatgen import Composition
+from pymatgen.core.composition import Composition
 from pymatgen.entries.computed_entries import ComputedEntry
 from pymatgen.analysis.phase_diagram import (
     PhaseDiagram,
@@ -26,7 +26,7 @@ from pymatgen.analysis.phase_diagram import (
     triangular_coord,
 )
 from pymatgen.analysis.structure_matcher import StructureMatcher
-from pymatgen import Structure
+from pymatgen.core.structure import Structure
 from camd.utils.data import cache_matrio_data, \
     filter_dataframe_by_composition, ELEMENTS
 from camd import CAMD_CACHE
@@ -406,7 +406,7 @@ class StabilityAnalyzer(AnalyzerBase):
         )
         # Drop excess columns from experiment
         new_seed = new_seed.drop([
-            'path', 'status', 'start_time', 'jobId', 'jobName',
+            'path', 'status', 'start_time', 'jobId', 'jobName', 'jobArn',
             'result', 'error', 'elapsed_time'
         ], axis="columns", errors="ignore")
         return summary, new_seed
@@ -503,7 +503,7 @@ class StabilityAnalyzer(AnalyzerBase):
             plotkwargs.update({"linestyle": "--"})
         else:
             plotkwargs.update({"linestyle": "-"})
-        plotter = PDPlotter(pd, **plotkwargs)
+        plotter = PDPlotter(pd, backend='matplotlib', **plotkwargs)
 
         getplotkwargs = {"label_stable": False} if finalize else {}
         plot = plotter.get_plot(**getplotkwargs)
@@ -520,7 +520,7 @@ class StabilityAnalyzer(AnalyzerBase):
             # measure stabilities wrt. the ultimate hull.
             pd = PhaseDiagram(filtered["entry"].values, elements=pg_elements)
             plotter = PDPlotter(
-                pd, **{"markersize": 0, "linestyle": "-", "linewidth": 2}
+                pd, backend="matplotlib", **{"markersize": 0, "linestyle": "-", "linewidth": 2}
             )
             plot = plotter.get_plot(plt=plot)
 
@@ -706,7 +706,7 @@ def update_run_w_structure(folder, hull_distance=0.2, parallel=True):
             old_results = df.drop(all_results.index, errors='ignore')
             new_results = df.drop(old_results.index)
             st_a = StabilityAnalyzer(
-                hull_distance=hull_distance, parallel=parallel, entire_space=True, plot=False)
+                hull_distance=hull_distance, parallel=parallel, entire_space=False, plot=False)
             summary, new_seed = st_a.analyze(new_results, old_results)
 
             # Having calculated stabilities again, we plot the overall hull.
@@ -729,7 +729,7 @@ def update_run_w_structure(folder, hull_distance=0.2, parallel=True):
                 unique_s_dict = {}
                 for i in range(len(s_a.structures)):
                     if s_a.structure_is_unique[i] and (
-                        s_a.structure_ids[i] in stable_discovered
+                        s_a.structure_ids[i] in stable_discovered.index
                     ):
                         unique_s_dict[s_a.structure_ids[i]] = s_a.structures[i]
 
