@@ -34,11 +34,18 @@ class CampaignTest(unittest.TestCase):
             new_campaign = Campaign(df, AgentStabilityML5(), ATFSampler(df),
                                     StabilityAnalyzer(), create_seed=10,
                                     s3_prefix="test")
-            new_campaign.initialize()
+            new_campaign.auto_loop(n_iterations=3, save_iterations=True, initialize=True)
+        # Test iteration read
         s3 = boto3.resource('s3')
         obj = s3.Object(CAMD_S3_BUCKET, "test/iteration.json")
         loaded = json.loads(obj.get()['Body'].read())
-        self.assertEqual(loaded, 0)
+        self.assertEqual(loaded, 2)
+
+        # Test save directories
+        for iteration in [-1, 0, 1, 2]:
+            obj = s3.Object(CAMD_S3_BUCKET, f"test/{iteration}/iteration.json")
+            loaded = json.loads(obj.get()['Body'].read())
+            self.assertEqual(loaded, iteration)
 
 
 if __name__ == '__main__':
