@@ -6,7 +6,6 @@ to be run asynchronously with a campaign.
 
 
 import os
-import uuid
 import json
 import re
 import time
@@ -17,11 +16,17 @@ import traceback
 import warnings
 from datetime import datetime
 
+import boto3
+from botocore.errorfactory import ClientError
+from tqdm import tqdm
+import pandas as pd
 from monty.os import cd
+from monty.tempfile import ScratchDir
 from pymatgen.io.vasp.outputs import Vasprun
-from pymatgen import Composition
+from pymatgen.core.composition import Composition
 from camd.experiment.base import Experiment
-from camd.utils.data import QMPY_REFERENCES, QMPY_REFERENCES_HUBBARD
+from camd.utils.data import QMPY_REFERENCES, QMPY_REFERENCES_HUBBARD, \
+    get_chemsys, get_common_prefixes
 
 
 class OqmdDFTonMC1(Experiment):
@@ -32,7 +37,9 @@ class OqmdDFTonMC1(Experiment):
     """
 
     def __init__(self, poll_time=60, timeout=7200, current_data=None, job_status=None,
-                 cleanup_directories=True):
+                 cleanup_directories=True, container_version="oqmdvasp/3",
+                 batch_queue="oqmd_test_queue", use_cached=False,
+                 prefix_append=None):
         """
         Initializes an OqmdDFTonMC1 instance
 
@@ -43,11 +50,43 @@ class OqmdDFTonMC1(Experiment):
             job_status (str): job status
             cleanup_directories (bool): flag to enable cleaning up of DFT results after
                 runs are over
+            container_version (str): container for mc1, e.g. "oqmdvasp/3" or "gpaw/1" which
+                dictates where things will be run
+            batch_queue (str): name of aws batch queue to submit to
+            use_cached (bool): whether to used cached results from prior campaigns
+            prefix_append (str): appended prefix to use for caching and data organization,
+                e.g. proto-dft-1
         """
-
         self.poll_time = poll_time
         self.timeout = timeout
         self.cleanup_directories = cleanup_directories
+        self.use_cached = use_cached
+        self.batch_queue = batch_queue
+
+        # Build parent directory using TRI_PATH and container/version
+        tri_path = os.environ.get("TRI_PATH")
+        tri_bucket = os.environ.get("TRI_BUCKET")
+        if not (tri_path and tri_bucket):
+            raise ValueError(
+                "TRI_PATH and TRI_BUCKET must be specified as env "
+                "variable to use camd MC1 interface"
+            )
+
+        container, version = container_version.split('/')
+        parent_dir = os.path.join(
+            tri_path,
+            "model",
+            container,
+            version,
+            "u",
+            "camd",
+        )
+        if prefix_append is not None:
+            if '_' in prefix_append:
+                raise ValueError("Prefix cannot contain underscores for mc1 compatibility")
+            parent_dir = os.path.join(parent_dir, prefix_append)
+        self.parent_dir = parent_dir
+
         super().__init__(current_data=current_data, job_status=job_status)
 
     def _update_job_status(self):
@@ -122,6 +161,7 @@ class OqmdDFTonMC1(Experiment):
             "start_time",
             "jobId",
             "jobName",
+            "jobArn",
             "result",
             "error",
             "delta_e",
@@ -182,30 +222,104 @@ class OqmdDFTonMC1(Experiment):
 
         return self.job_status
 
+    def fetch_cached(self, candidate_data):
+        """
+        Fetches cached data based on candidate data
+
+        Args:
+            candidate_data (pd.DataFrame): Pandas dataframe
+
+        Returns:
+            (pandas.DataFrame) dataframe with data filled out
+
+        """
+        tri_path = os.environ.get("TRI_PATH")
+        tri_bucket = os.environ.get("TRI_BUCKET")
+        s3_client = boto3.client("s3")
+        # Scrub tri_path and replace model with simulation
+        # to get s3 key
+        s3_parent = self.parent_dir.replace('model', 'simulation')
+        s3_parent = s3_parent.replace(tri_path + '/', "")
+        cached_experiments = pd.DataFrame()
+        # Get all experiment folders
+        chemsyses = set([get_chemsys(s) for s in candidate_data['structure']])
+        experiment_dirs = []
+        for chemsys in chemsyses:
+            chemsys_dirs = get_common_prefixes(
+                tri_bucket,
+                os.path.join(s3_parent, chemsys)
+            )
+            experiment_dirs.extend(chemsys_dirs)
+        for structure_id, row in tqdm(candidate_data.iterrows(), total=len(candidate_data)):
+            if not structure_id.replace('-', '') in experiment_dirs:
+                continue
+            calc_path = os.path.join(
+                s3_parent, get_chemsys(row['structure']),
+                structure_id.replace('-', ''), "_1/")
+            with ScratchDir('.'):
+                # Figure out whether prior submission exists
+                cached_experiments = cached_experiments.append(row)
+                # TODO: figure out whether file exists in s3
+                # TODO: this is a little crude, could use boto3
+                try:
+                    # import pdb; pdb.set_trace()
+                    vr_path = os.path.join(calc_path, "static", "vasprun.xml")
+                    cmd = "aws s3 cp s3://{}/{} .".format(tri_bucket, vr_path)
+                    subprocess.call(shlex.split(cmd))
+                    vr = Vasprun("vasprun.xml")
+                    vr_dict = vr.as_dict()
+                    delta_e = get_qmpy_formation_energy(
+                        vr_dict["output"]["final_energy_per_atom"],
+                        vr_dict["pretty_formula"],
+                        1,
+                    )
+                    data = {
+                        "status": "SUCCEEDED",
+                        "error": None,
+                        "result": vr,
+                        "delta_e": delta_e,
+                    }
+                except Exception as e:
+                    error_doc = {}
+                    try:
+                        err_obj = s3_client.get_object(
+                            Bucket=tri_bucket, Key=os.path.join(calc_path, 'err'))
+                        errtxt = err_obj['Body'].read().decode('utf-8')
+                        error_doc.update(
+                            {"mc1_stderr": errtxt}
+                        )
+                    except ClientError:
+                        print('No error file for {}'.format(calc_path))
+                    error_doc.update(
+                        {
+                            "camd_exception": "{}".format(e),
+                            "camd_traceback": traceback.format_exc(),
+                        }
+                    )
+                    # Dump error docs to avoid Pandas issues with dict values
+                    data = {"status": "FAILED", "error": json.dumps(error_doc)}
+                update_dataframe_row(cached_experiments, structure_id, data, add_columns=True)
+        return cached_experiments
+
     def submit_dft_calcs_to_mc1(self):
         """
         Helper method for submitting DFT calculations to MC1
         """
-        tri_path = os.environ.get("TRI_PATH")
-        if not tri_path:
-            raise ValueError(
-                "TRI_PATH must be specified as env variable to "
-                "use camd MC1 interface"
-            )
 
         # Create run directory
-        uuid_string = str(uuid.uuid4()).replace("-", "")
-        parent_dir = os.path.join(
-            tri_path, "model", "oqmdvasp", "3", "u", "camd", "run{}".format(uuid_string)
-        )
+        # uuid_string = str(uuid.uuid4()).replace("-", "")
+
         if any(["_" in value for value in self.current_data.index]):
             raise ValueError(
                 "Structure keys cannot contain underscores for " "mc1 compatibility"
             )
 
         for structure_id, row in self.current_data.iterrows():
-            calc_path = os.path.join(parent_dir, structure_id, "_1")
-            os.makedirs(calc_path)
+            # Replace structure id in path to avoid confusing mc1
+            calc_path = os.path.join(
+                self.parent_dir, get_chemsys(row['structure']),
+                structure_id.replace('-', ''), "_1")
+            os.makedirs(calc_path, exist_ok=True)
             with cd(calc_path):
                 # Write input cif file and python model file
                 row["structure"].to(filename="POSCAR")
@@ -220,7 +334,7 @@ class OqmdDFTonMC1(Experiment):
                         [
                             "trisub",
                             "-q",
-                            "oqmd_test_queue",
+                            self.batch_queue,
                             "-r",
                             "16000",
                             "-c",
@@ -229,19 +343,22 @@ class OqmdDFTonMC1(Experiment):
                             "us-east-1",
                         ]
                     )
+                    response = response.decode("utf-8")
+                    response = re.findall("({.+})", response, re.DOTALL)[0]
+                    data = json.loads(response)
+                    data.update(
+                        {
+                            "path": os.getcwd(),
+                            "status": "SUBMITTED",
+                            "start_time": datetime.utcnow(),
+                        }
+                    )
                 except subprocess.CalledProcessError as e:
                     print(e.output)
+                    data = {"path": os.getcwd(),
+                            "status": "FAILED",
+                            "error": "failed submission {}".format(e.output)}
 
-                response = response.decode("utf-8")
-                response = re.findall("({.+})", response, re.DOTALL)[0]
-                data = json.loads(response)
-                data.update(
-                    {
-                        "path": os.getcwd(),
-                        "status": "SUBMITTED",
-                        "start_time": datetime.utcnow(),
-                    }
-                )
                 update_dataframe_row(self.current_data, structure_id, data)
 
     def check_dft_calcs(self):
@@ -281,8 +398,9 @@ class OqmdDFTonMC1(Experiment):
                     }
                 except Exception as e:
                     error_doc = {}
-                    with open("err") as errfile:
-                        error_doc.update({"trisub_stderr": errfile.read()})
+                    if os.path.isfile("err"):
+                        with open("err") as errfile:
+                            error_doc.update({"mc1_stderr": errfile.read()})
                     error_doc.update(
                         {
                             "camd_exception": "{}".format(e),
@@ -390,7 +508,8 @@ def get_qmpy_formation_energy(total_e, formula, n_atoms):
     return energy
 
 
-def update_dataframe_row(dataframe, index, update_dict):
+def update_dataframe_row(dataframe, index, update_dict,
+                         add_columns=False):
     """
     Method to update a dataframe row via an update_dictionary
     and an index, similarly to Dict.update()
@@ -400,10 +519,14 @@ def update_dataframe_row(dataframe, index, update_dict):
             be updated
         index: index value for dataframe
         update_dict ({}): update dictionary for dataframe
+        add_columns (bool): whether to add non-existent
+            columns to the dataframe
 
     Returns:
         None
 
     """
     for key, value in update_dict.items():
+        if add_columns and key not in dataframe.columns:
+            dataframe[key] = None
         dataframe.loc[index, key] = value
